/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.listener;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.Constants.Interceptor.Phase;
import org.ow2.petals.component.framework.api.Interceptor;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.interceptor.MessageInterceptorManager;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.message.ExchangeImpl;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;
import org.ow2.petals.component.framework.su.ServiceUnitManager;

/**
 * The message interceptor processor. Used by the component to intercept
 * messages.
 * 
 * @author chamerling - EBM WebSourcing
 * @since 2.0
 * 
 */
public class MessageInterceptorProcessor {

    /**
     * The JBI component instance
     */
    protected AbstractComponent component;

    /**
     * The logger
     */
    protected Logger logger;

    /**
     * Creates a new instance of {@link MessageInterceptorProcessor}
     * 
     */
    public MessageInterceptorProcessor(AbstractComponent component, Logger logger) {
        this.component = component;
        this.logger = logger;
    }

    /**
     * Intercepts the message.
     * 
     * @param exchange
     * @param extensible
     */
    public void intercept(final Exchange exchange, final Object service, final Phase phase)
            throws PEtALSCDKException {
        logger.fine("Intercepting message processing on " + phase);

        this.checkPhase(phase, service);

        ServiceUnitManager suManager = (ServiceUnitManager) component.getServiceUnitManager();

        ServiceUnitDataHandler dataHandler = suManager.getSUDataHandlerForService(service);

        if (dataHandler != null) { // no DH when no deployed SU
            MessageInterceptorManager messageInterceptorManager = dataHandler
                    .getMessageInterceptorManager(service);

            // unactivate the check on the Role when intercepting at
            // reception,
            // to be able to reset the normalized message.
            if (Phase.ACCEPT.equals(phase)) {
                ((ExchangeImpl) exchange).setCheckRole(false);
            }
            this.intercept(exchange, service, messageInterceptorManager, phase);
            if (Phase.ACCEPT.equals(phase)) {
                ((ExchangeImpl) exchange).setCheckRole(true);
            }
        }
    }

    private void checkPhase(Phase phase, Object service) throws PEtALSCDKException {
        if (Phase.ACCEPT.equals(phase) || Phase.ACCEPT_RESPONSE.equals(phase)) {
            if (!(service instanceof Provides)) {
                throw new PEtALSCDKException("The phase is not a Provides phase");
            }
        } else {
            if (!(service instanceof Consumes)) {
                throw new PEtALSCDKException("The phase is not a Consumes phase");
            }
        }
    }

    /**
     * 
     * @param exchange
     * @param extensible
     * @param messageInterceptorManager
     */
    private void intercept(Exchange exchange, Object service,
            MessageInterceptorManager messageInterceptorManager, Phase phase) {
        Map<String, Interceptor> interceptors = null;
        Map<String, ConfigurationExtensions> additionalConfiguration = null;

        // getting good interceptors
        if (phase == Phase.ACCEPT) {
            interceptors = messageInterceptorManager.getInputOrderedInterceptors();
            additionalConfiguration = messageInterceptorManager.getInputAdditionalConfiguration();
        } else if (phase == Phase.SEND) {
            interceptors = messageInterceptorManager.getOutputOrderedInterceptors();
            additionalConfiguration = messageInterceptorManager.getOutputAdditionalConfiguration();
        } else if (phase == Phase.ACCEPT_RESPONSE) {
            interceptors = messageInterceptorManager.getInputResponseOrderedInterceptors();
            additionalConfiguration = messageInterceptorManager
                    .getInputResponseAdditionalConfiguration();
        } else if (phase == Phase.SEND_RESPONSE) {
            interceptors = messageInterceptorManager.getOutputResponseOrderedInterceptors();
            additionalConfiguration = messageInterceptorManager
                    .getOutputResponseAdditionalConfiguration();
        }

        for (String interceptorName : interceptors.keySet()) {
            logger.fine("Calling message interceptor " + interceptorName);

            // get interceptor
            Interceptor interceptor = interceptors.get(interceptorName);

            // override component parameters with SU ones
            Map<String, String> finalParameters = new HashMap<String, String>();
            finalParameters.putAll(interceptor.getComponentParameters());
            if (additionalConfiguration.get(interceptorName) != null) {
                finalParameters.putAll(additionalConfiguration.get(interceptorName));
            }

            // intercept...
            if (phase == Phase.ACCEPT) {
                logger.fine("Intercepting message on accept phase");
                interceptor.handleMessageAccept(exchange, (Provides) service, finalParameters);
            } else if (phase == Phase.SEND) {
                logger.fine("Intercepting message on send phase");
                interceptor.handleMessageSent(exchange, (Consumes) service, finalParameters);
            } else if (phase == Phase.ACCEPT_RESPONSE) {
                logger.fine("Intercepting message on accept response phase");
                interceptor.handleMessageAcceptResponse(exchange, (Provides) service,
                        finalParameters);
            } else if (phase == Phase.SEND_RESPONSE) {
                logger.fine("Intercepting message on send response phase");
                interceptor
                        .handleMessageSentResponse(exchange, (Consumes) service, finalParameters);
            }
        }
    }
}
