/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.performance;

import javax.management.Notification;
import javax.management.NotificationBroadcasterSupport;
import javax.management.ObjectName;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Component;
import org.ow2.petals.component.framework.util.IDGenerator;

/**
 * The MBean used to manage the performance notifications via JMX.
 * 
 * @author cdeneux - Capgemini Sud
 * @since 2.0
 * 
 */
public class PerformanceNotifier extends NotificationBroadcasterSupport implements PerformanceNotifierMBean {

    /**
     * Name of the normalized message containing the performance notification
     * UID.
     */
    public static final String PERF_NOTIF_NMR_MSG_ATTR = "petals:perfNotifUID";

    /**
     * Performance notification name.
     */
    private static final String ATTR_NOTIFICATIONS_VALUES = "performanceNotificationData";
    
    /**
     * The MBean name of this MBean instance
     */
    private ObjectName mbeanName;

    /** JMX sequence numbers and their mutex */
    private PerformanceNotificationCounter counters;

    private long jmxGlobalSeqNum;

    private Object jmxGlobalSeqNumObj;

    private Component componentConfiguration;

    /**
     * Creates a new instance of {@link PerformanceNotifier}
     * 
     * @param componentConfiguration
     * @param jmxPerformanceNotifierName
     */
    public PerformanceNotifier(Component componentConfiguration, ObjectName mbeanName) {
        this.componentConfiguration = componentConfiguration;
        this.jmxGlobalSeqNumObj = new Object();
        this.mbeanName = mbeanName;
        if (this.componentConfiguration.getPerformanceStep() == null) {
            this.counters = new PerformanceNotificationCounter(1);
        } else {
            this.counters = new PerformanceNotificationCounter(this.componentConfiguration
                    .getPerformanceStep().getValue());
        }
    }

    /**
     * Creates a performance notification.
     * 
     * @param timestamp
     * @param owner
     * @param typeNotif
     * @param userData
     * @param msgExchange
     * @return
     */
    public final PerformanceNotification createPerformanceNotifications(final long timestamp,
            final String owner, final String typeNotif, final String userData, Exchange msgExchange) {

        PerformanceNotification performanceNotification = null;
        if (counters.inc(typeNotif)) {
            // We try to find a performance notification message exists in the
            // exchange, or create a new one
            String perfUID = null;
            if (msgExchange != null) {
                // we try to find a performance notification UID
                perfUID = (String) msgExchange.getProperty(PERF_NOTIF_NMR_MSG_ATTR);
            }

            if (perfUID == null) {
                perfUID = IDGenerator.getInstance().getNewID();
                if (msgExchange != null) {
                    msgExchange.setProperty(PERF_NOTIF_NMR_MSG_ATTR, perfUID);
                }
            }

            synchronized (jmxGlobalSeqNumObj) {
                performanceNotification = new PerformanceNotification(timestamp, owner, typeNotif,
                        userData, ++jmxGlobalSeqNum, perfUID);
            }
        }

        return performanceNotification;
    }

    /**
     * Publish the performance notification
     * 
     * @param notification
     */
    public final void send(final PerformanceNotification notification) {

        if (notification != null) {
            notification.end();

            // Notification creation
            Notification jmxNotification = new Notification(ATTR_NOTIFICATIONS_VALUES, this
                    .getClass().getName(), notification.getGlobalSeqNum());

            String[] userData = { notification.getUID(),
                    Long.toString(notification.getTimestamp()), notification.getOwner(),
                    notification.getType(), Long.toString(notification.getDuration()),
                    notification.getUserData() };

            jmxNotification.setUserData(userData);

            // Notification sending
            this.sendNotification(jmxNotification);
        }
    }
   
    /**
     * Get the MBean Name
     * 
     * @return
     */
    public ObjectName getMBeanName() {
        return this.mbeanName;
    }
}
